--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MCLSHIFT
--

library ieee;
use ieee.std_logic_1164.all;

package Mclshift_local is
    FUNCTION log2 (val : natural) RETURN natural ;
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
    component Mcompare
	generic(lpm_width	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	std_logic_vector(lpm_width - 1 downto 0);
		dataB		: in	std_logic_vector(lpm_width - 1 downto 0);
		alb		: out	std_logic;
		aeb		: out	std_logic;
		agb		: out	std_logic;
		ageb		: out   std_logic;
		aleb		: out   std_logic;
		aneb		: out   std_logic);
    end component;
component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
end Mclshift_local ;

package body Mclshift_local is
    function port_state (arg : std_logic) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	result := arg'port_state_att;

	return result;
    end port_state;

    function connects (x : std_logic; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

    function vectorize (a, size : natural) return std_logic_vector is
	variable result : std_logic_vector(size-1 downto 0) := (others => '0');
	variable b : natural;
	variable newsize : integer := size;
    begin
	if (newsize > 32) then 
	    newsize := 32;
	    assert (false)
	    report "VECTORIZE: Widths > 32 not allowed, vector truncated"
	    severity warning;
	end if;
	if (newsize <= 31) then
	    for i in result'reverse_range loop
		if ((a/(2**i)) mod 2) = 1 then
		    result(i) := '1';
		end if;
	    end loop;
	end if;
	if (newsize = 32) then
	    b := a/2;
	    if ((a mod 2) = 1) then
		result(result'low) := '1';
	    end if;
	    for i in result'low to (result'high - 1) loop
		if ((b/(2**i)) mod 2) = 1 then
		    result(i+1) := '1';
		end if;
	    end loop;
	end if;
	if newsize >= 31 then
	    if ((2**(newsize-2)) < (a/4)) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	else
	    if ((2**newsize) < a) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	end if;
	return result;
    end vectorize;
    FUNCTION log2 (val : natural) RETURN natural IS
	variable tmp  : natural := val;
	variable sval : natural;
    BEGIN
	if (tmp REM 2) = 0 then tmp := tmp - 1; end if;
	sval := 0;
	while tmp > 0 loop
	    sval := sval + 1;
	    tmp := tmp / 2;
	end loop;
	return sval;
    END log2;

end Mclshift_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Mclshift_local.all;

entity Mclshift is
    generic(lpm_width		: positive;
	    lpm_widthdist	: natural;
	    lpm_shifttype	: shift_type;
	    lpm_hint		: goal_type);
    port(   data		: in  std_logic_vector((lpm_width-1) downto 0);
	    distance		: in  std_logic_vector((lpm_widthdist-1) downto 0);
	    direction 		: in  std_logic;
	    result		: out std_logic_vector((lpm_width-1) downto 0);
	    overflow		: out std_logic;
	    underflow		: out std_logic);
end Mclshift;

architecture archlpmstd of Mclshift is

    constant movement : natural := log2(lpm_width);
    constant shift : natural := max(movement,lpm_widthdist);
    signal def : std_logic;
    signal OK : boolean := true;
    signal widthv : std_logic_vector(distance'range);
    signal shifter,ors,dist : std_logic_vector((shift-1) downto 0) := (others => '0');
    signal barrel:std_logic_vector(((result'length*result'length)-1) downto 0);
    signal left,right,sign,mask : std_logic_vector(result'high downto 0);
    signal output,ioutput,loutput,aoutput : std_logic_vector(result'high downto 0);
    signal lunderflow,loverflow,aunderflow,aoverflow : std_logic;
    signal un,ov : std_logic_vector(result'high downto 0);

begin

    CP: if (2**lpm_widthdist > lpm_width) generate
	widthv <= vectorize(lpm_width,lpm_widthdist);
	U0: mcompare generic map (lpm_widthdist,lpm_unsigned,lpm_hint)
			port map (distance,widthv,def,open,open,open,open,open);
	OK <= true when (def = '1') else false;
    end generate;

    dist((lpm_widthdist-1) downto 0) <= distance;
    shifter(0) <= dist(0);
    ors(0) <= dist(0);
    GS: for i in (shifter'low + 1) to shifter'high generate
	ors(i) <= ors(i-1) OR dist(i);
	shifter(i) <= dist(i) when (direction = '1')
				    else dist(i) XOR (ors(i-1));
    end generate;

    GB: for j in data'low to data'high generate
	GJ1: for k in data'low to data'high generate
	    barrel((j*data'length)+k) <= data((j+k) rem result'length);
	end generate;
    end generate;

    -- Multiplex the 'barrel' shifter inputs using 'shifter' as the selector
    process (barrel, shifter)
	variable tmp : std_logic_vector(result'high downto 0) ;
    begin
	tmp := (others => '0') ;
	for selval in 0 to ((2 ** movement)-1) loop
	    if (result'length*(selval+1) <= result'length*result'length) AND
				    (vectorize(selval, shift) = shifter) then
		tmp := barrel(((result'length * (selval+1))-1) downto (result'length * selval)) ;
	    end if ;
	end loop ;
	ioutput <= tmp ;
    end process ;

    B: for i in output'low to output'high generate
	B1: buf port map (ioutput(i),output(i));
    end generate;

    GM:if (lpm_shifttype /= lpm_rotate) generate
	GML: for i in result'low to result'high generate
	    D0:if (i <  lpm_widthdist**2) generate
		left(i) <= '1' when
		    ((vectorize(i,lpm_widthdist)) < distance) else '0';
		right(result'high-i) <= '1' when
		    ((vectorize(i,lpm_widthdist)) < distance) else '0';
	    end generate;
	    D1:if (i >= lpm_widthdist**2) generate
		left(i) <= '0';
		right(result'high-i) <= '0';
	    end generate;
	end generate;
	mask <= left when (direction = '0') else right;
    end generate;

    GL:if (lpm_shifttype = lpm_logical) generate
	loutput <= (output AND NOT mask);

	ov(0) <= output(0) AND mask(0);
	GLO: for i in (result'low + 1) to result'high generate
	    ov(i) <= ov(i-1) OR (output(i) AND mask(i));
	end generate;
	loverflow <= NOT direction AND ov(ov'high);

	un(0) <= loutput(0);
	GLU: for i in (result'low + 1) to result'high generate
	    un(i) <= un(i-1) OR loutput(i);
	end generate;
	lunderflow <= direction AND NOT un(un'high);
    end generate;

    GA:if (lpm_shifttype = lpm_arithmetic) generate
	sign <= (OTHERS => data(data'high));
	L0: for i in result'low to (result'high - 1) generate
	    aoutput(i) <= output(i) AND NOT mask(i) when (direction = '0')
		else (output(i) AND NOT mask(i)) OR (right(i) AND sign(i));
	end generate;
	aoutput(result'high) <= data(result'high) when (direction = '0')
	    else (output(result'high) AND NOT mask(result'high))
	    OR (right(result'high) AND sign(result'high));

	ov(0) <= (output(0) XOR sign(0)) AND mask(0);
	GAO: for i in (result'low + 1) to result'high generate
	    ov(i) <= ov(i-1) OR ((output(i) XOR sign(i)) AND mask(i));
	end generate;
	aoverflow <= NOT direction AND
		(ov(ov'high) OR (output(output'high) XOR data(data'high)));

	un(0) <= aoutput(0) XOR NOT sign(0);
	GAU: for i in (result'low + 1) to result'high generate
	    un(i) <= un(i-1) AND (aoutput(i) XOR NOT sign(i));
	end generate;
	aunderflow <= direction AND un(un'high);
    end generate;

    result    <= aoutput when ((lpm_shifttype = lpm_arithmetic) AND OK) else
		 loutput when ((lpm_shifttype = lpm_logical) AND OK) else
		 output  when  (lpm_shifttype = lpm_rotate) else
		 (OTHERS => data(data'high))
		 when ((lpm_shifttype = lpm_arithmetic) AND NOT OK) else
		 (OTHERS => '0');
    overflow  <= aoverflow  when (lpm_shifttype = lpm_arithmetic) else
		 loverflow  when (lpm_shifttype = lpm_logical) else '0';
    underflow <= aunderflow when (lpm_shifttype = lpm_arithmetic) else
		 lunderflow when (lpm_shifttype = lpm_logical) else '0';

end archlpmstd;
