--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MCOUNTER
--

library ieee;
use ieee.std_logic_1164.all;

package Mcounter_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
component tsrff
    port(
	t	: in std_logic;		-- T input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
end Mcounter_local ;

package body Mcounter_local is
    function port_state (arg : std_logic) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	result := arg'port_state_att;

	return result;
    end port_state;

    function connects (x : std_logic; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

    function restring (a : string; w : positive) return std_logic_vector is
	variable result : std_logic_vector (w-1 downto 0) := (others => '1');
	variable indx : natural;
    begin

	if (a'length = 0) then
	    return result;
	elsif (a'length /= w) then
	    Assert (false)
	    Report "RESTRING: Source string length /= target vector width."
	    Severity Failure;
	    return result;
	else
	    indx := 0;
	    for i in a'reverse_range loop
		case a(i) is
		when '1' => result(indx) := '1';
		when '0' => result(indx) := '0';

		when '-' => result(indx) := '-';

		when others => Assert (false)

		    Report "Invalid char. literal (must be '1', '0', or '-').  Defauting to  '0'."

		    Severity Warning;
		    result(indx) := '0';
		end case;
		indx := indx + 1;
	    end loop;
	    return result;
	end if;
    end restring;
end Mcounter_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Mcounter_local.all;

entity Mcounter is
    generic(lpm_width	: positive;
	    lpm_direction	: ctdir_type;
	    lpm_avalue	: string;
	    lpm_svalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	std_logic_vector((lpm_width-1) downto 0);
	    clock	: in	std_logic;
	    clk_en	: in	std_logic;
	    cnt_en	: in	std_logic;
	    updown	: in	std_logic;
	    q		: out	std_logic_vector((lpm_width-1) downto 0);
	    aset	: in	std_logic;
	    aclr	: in	std_logic;
	    aload	: in	std_logic;
	    sset	: in	std_logic;
	    sclr	: in	std_logic;
	    sload	: in	std_logic;
	    testenab	: in	std_logic;
	    testin	: in	std_logic;
	    testout	: out	std_logic);
end Mcounter;

architecture archlpmstd of Mcounter is

    signal qtmp,t,en_up,en_dn,en,enu : std_logic_vector(q'high downto 0);
    signal async,sync,set,clr : std_logic_vector(q'high downto 0);

    constant PIM: integer := 36;
    constant N	: integer := connects(aset,PIM);
    constant O	: integer := connects(aclr,N);
    constant P	: integer := connects(sset,O);
    constant R	: integer := connects(sclr,P);
    constant S	: integer := connects(sload,R);
    constant U	: integer := connects(sload,S);
    constant V	: integer := connects(testenab,U);
    constant W	: integer := connects(clk_en,V);
    constant X	: integer := connects(cnt_en,W);
    constant Y	: integer := connects(updown,X);

    attribute ff_type of qtmp:signal is ff_t ;

begin

    async <= restring(lpm_avalue,lpm_width);
    sync <= restring(lpm_svalue,lpm_width);

    en_up(0) <= '1';
    en_dn(0) <= '1';
    en(0) <= enu(0);
    G0: for j in q'range generate

	G1:if (j /= 0) generate
	    TU: if (((port_state(updown) = ps_tied_high)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_up)) generate
		en_up(j) <= (en(j-1) AND     qtmp(j-1));
	    end generate;
	    TD: if (((port_state(updown) = ps_tied_low)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_down)) generate
		en_dn(j) <= (en(j-1) AND NOT qtmp(j-1));
	    end generate;
	    TB: if ((port_state(updown) = ps_connected)
		     AND (lpm_direction = lpm_no_dir)) generate
		en_up(j) <= (en(j-1) AND     updown AND     qtmp(j-1));
		en_dn(j) <= (en(j-1) AND NOT updown AND NOT qtmp(j-1));
	    end generate;
	end generate;
	
	G2: if (((lpm_hint = AREA) AND ((j rem (Y-8)) /= 0)) OR
		((lpm_hint /= AREA) AND ((j rem Y) /= 0))) generate
	    TU: if (((port_state(updown) = ps_tied_high)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_up)) generate
		en(j) <= en_up(j);
	    end generate;
	    TD: if (((port_state(updown) = ps_tied_low)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_down)) generate
		en(j) <= en_dn(j);
	    end generate;
	    TB: if ((port_state(updown) = ps_connected)
		     AND (lpm_direction = lpm_no_dir)) generate
		en(j) <= en_up(j) OR en_dn(j);
	    end generate;
	end generate;

	G3: if (((lpm_hint = AREA) AND ((j rem (Y-8)) = 0)) OR
		((lpm_hint /= AREA) AND ((j rem Y) = 0))) generate
	    TU: if (((port_state(updown) = ps_tied_high)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_up)) generate
		enu(j) <= en_up(j);
	    end generate;
	    TD: if (((port_state(updown) = ps_tied_low)
		      AND (lpm_direction = lpm_no_dir))
		       OR (lpm_direction = lpm_down)) generate
		enu(j) <= en_dn(j);
	    end generate;
	    TB: if ((port_state(updown) = ps_connected)
		     AND (lpm_direction = lpm_no_dir)) generate
		enu(j) <= en_up(j) OR en_dn(j);
	    end generate;
	    XB: if (j /= 0) generate
		BJ: buf port map (enu(j), en(j));
	    end generate;
	end generate;
	
	set(j) <= (aset AND     async(j)) OR (aload AND     data(j));
	clr(j) <= (aset AND NOT async(j)) OR (aload AND NOT data(j)) OR aclr;

	UJ: tsrff port map(t(j),set(j),clr(j),clock,qtmp(j));

    end generate;

    process (data,sset,sclr,sload,en,cnt_en,testin,testenab,qtmp)
    begin
	if (testenab = '1') then
	    t(0) <= testin XOR qtmp(0);
	    if (lpm_width > 1) then
		t(q'high downto 1) <= qtmp(q'high downto 1) XOR
				      qtmp((q'high - 1) downto 0);
	    end if;
	elsif (sclr = '1' AND clk_en = '1') then
	    t <= qtmp;
	elsif (sset = '1' AND clk_en = '1') then
	    t <= sync XOR qtmp;
	elsif (sload = '1' AND clk_en = '1') then
	    t <= data XOR qtmp;
	elsif (cnt_en = '1' AND clk_en = '1') then
	    t <= en;
	else
	    t <= (others => '0');
	end if;
    end process;

    testout <= qtmp(q'high);
    q <= qtmp;

end archlpmstd;
