--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--
--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    COMPONENT bufoe		-- Tri-state input/output with feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: inout std_logic;		-- std_logic output pin.
	    yfb	: OUT std_logic		-- Output feed back as std_logic.
	);
    END COMPONENT;
    attribute atomic_rtl of bufoe : component is bufoe_type;

    COMPONENT dltch		-- Transparent D latch.
	PORT (
	    d	: IN std_logic;		-- D input to latch.
	    e	: IN std_logic;		-- Enabel, tranparent high.
	    q	: OUT std_logic		-- Qutput from latch.
	);
    END COMPONENT;
    attribute atomic_rtl of dltch : component is dltch_type;

    COMPONENT dff		-- D type flip flop. (only D and clk input)
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dff : component is dff_type;

    COMPONENT xdff		-- X type flip flop. (X1, X2 and clk input)
	PORT (
	    x1	: IN std_logic;		-- First Exclusive OR input.
	    x2	: IN std_logic;		-- Second Exclusive OR input.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT jkff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN std_logic;		-- J input to J-K register.
	    k	: IN std_logic;		-- K input to J-K register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT buf		-- Buffer
	PORT (
	    x	: IN std_logic;		-- Input to Buffer.
	    y	: OUT std_logic		-- Output from Buffer.
	);
    END COMPONENT;
    attribute atomic_rtl of buf : component is buf_type;

    COMPONENT srlch		-- Set, Reset Latch. (S and R input only)
	PORT (
	    s	: IN std_logic;		-- Set input, Async.
	    r	: IN std_logic;		-- Reset input, Async.
	    q	: OUT std_logic		-- Output from Latch.
	);
    END COMPONENT;

    COMPONENT srff		-- Synchronous Set Reset Flip Flop.
	PORT (
	    s	: IN std_logic;		-- Set input, Sync.
	    r	: IN std_logic;		-- Reset inut, Sync.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
--    attribute atomic_rtl of srff : component is srff_type;
-- 	This is being "fixed" in the library for Release 1.1.
--	It should be really fixed in the fitter.

    COMPONENT dsrff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dsrff : component is dsrff_type;

    COMPONENT tsrff		-- T type flip flop with async. set and reset.
	PORT (
	    t	: IN std_logic;		-- T input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of tsrff : component is tsrff_type;

    COMPONENT tff		-- T type flip flop. (T and clk input)
	PORT (
	    t	: IN std_logic;		-- Togal input to T type register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of tff : component is tff_type;

    COMPONENT xbuf		-- Xor primitive gate
	PORT (
	    x1	: IN std_logic;		-- Xor input.
	    x2	: IN std_logic;		-- Xor input.
	    q	: OUT std_logic		-- Xor output.
	);
    END COMPONENT;

    COMPONENT triout		-- Tri-state output with no feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: OUT std_logic		-- std_logic output pin.
	);
    END COMPONENT;
    attribute atomic_rtl of triout : component is triout_type;

    component idff
	port(
	    d_i   : in std_logic;		-- D input to input register.
	    clk_i : in std_logic;		-- Positive edge, input reg, clock.
	    q_i   : out std_logic		-- Output from input register.
	);
    end component;
    attribute atomic_rtl of idff : component is idff_type;

END rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------
--
-- FOR Release 1.1 ONLY.

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN std_logic;		-- First Exclusive OR input.
	r	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END srff;

ARCHITECTURE archsrff OF srff IS
    SIGNAL dx,qx : std_logic;
BEGIN
    dx	<= (s AND NOT r) OR (NOT r AND qx);
    u1 :dff port map(dx,clk,qx);
    q <= qx;
END archsrff;
-- FOR Release 1.1 ONLY.
--

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xdff IS
    PORT (
	x1	: IN std_logic;		-- First Exclusive OR input.
	x2	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END xdff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s	: IN std_logic;		-- Set input, Async.
	r	: IN std_logic;		-- Reset input, Async.
	q	: OUT std_logic		-- Output from Latch.
    );
END srlch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xbuf IS
    PORT (
	x1	: IN std_logic;		-- Xor input.
	x2	: IN std_logic;		-- Xor input.
	q	: OUT std_logic		-- Xor output.
    );
END xbuf;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archxdff OF xdff IS
    SIGNAL dx : std_logic;
BEGIN
    dx	<= x1 xor x2;
    u1 :dff port map(dx,clk,q);
END archxdff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archxbuf OF xbuf IS
BEGIN
    q	<= x1 xor x2;
END archxbuf;

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY xor2 IS
    port(
	a,b     : in std_logic;
	q       : out std_logic);
END xor2;
			 
ARCHITECTURE archXOR2 OF xor2 IS
begin
    u1 : xbuf port map (a, b, q);
END archXOR2;

------------------------------------------------------------------
--		Entities for all CPLD parts:			--
--		  371, 372, 373, 374, 375, 376.			--
------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity c371 is
    port (
	pin2	: inout std_logic;  
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: in    std_logic;
	pin11	: in    std_logic;
	pin13	: in    std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin21	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin32	: in    std_logic;
	pin33	: in    std_logic;
	pin35	: in    std_logic;
	pin36	: inout std_logic;    
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: inout std_logic;
	pin42	: inout std_logic;
	pin43	: inout std_logic);
end C371;

library ieee;
use ieee.std_logic_1164.all;

entity c372 is
    port (
	pin2	: inout std_logic;  
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: in    std_logic;
	pin11	: in    std_logic;
	pin13	: in    std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin21	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin32	: in    std_logic;
	pin33	: in    std_logic;
	pin35	: in    std_logic;
	pin36	: inout std_logic;    
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: inout std_logic;
	pin42	: inout std_logic;
	pin43	: inout std_logic);
end C372;

library ieee;
use ieee.std_logic_1164.all;

entity c373 is
    port (
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: inout std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: in    std_logic;
	pin23	: in    std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin33	: inout std_logic;    
	pin34	: inout std_logic;    
	pin35	: inout std_logic;    
	pin36	: inout std_logic;    
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: in    std_logic;
	pin45	: inout std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin50	: inout std_logic;
	pin51	: inout std_logic;
	pin52	: inout std_logic;
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: inout std_logic;
	pin60	: inout std_logic;
	pin61	: inout std_logic;
	pin62	: in    std_logic;
	pin65	: in    std_logic;
	pin66	: inout std_logic;
	pin67	: inout std_logic;
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin71	: inout std_logic;
	pin72	: inout std_logic;
	pin73	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: inout std_logic;
	pin79	: inout std_logic;
	pin80	: inout std_logic;
	pin81	: inout std_logic;
	pin82	: inout std_logic;
	pin83	: in    std_logic);
end C373;

library ieee;
use ieee.std_logic_1164.all;

entity c374 is
    port (
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: inout std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: in    std_logic;
	pin23	: in    std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin33	: inout std_logic;    
	pin34	: inout std_logic;    
	pin35	: inout std_logic;    
	pin36	: inout std_logic;    
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: in    std_logic;
	pin45	: inout std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin50	: inout std_logic;
	pin51	: inout std_logic;
	pin52	: inout std_logic;
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: inout std_logic;
	pin60	: inout std_logic;
	pin61	: inout std_logic;
	pin62	: in    std_logic;
	pin65	: in    std_logic;
	pin66	: inout std_logic;
	pin67	: inout std_logic;
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin71	: inout std_logic;
	pin72	: inout std_logic;
	pin73	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: inout std_logic;
	pin79	: inout std_logic;
	pin80	: inout std_logic;
	pin81	: inout std_logic;
	pin82	: inout std_logic;
	pin83	: in    std_logic);
end C374;

library ieee;
use ieee.std_logic_1164.all;

entity c375 is
    port (
	pin2	: inout std_logic;  
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin11	: inout std_logic;  
	pin12	: inout std_logic;  
	pin13	: inout std_logic;  
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: in std_logic;
	pin22	: in std_logic;
	pin23	: inout std_logic;  
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin32	: inout std_logic;  
	pin33	: inout std_logic;  
	pin34	: inout std_logic;
	pin35	: inout std_logic;
	pin36	: inout std_logic;
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin42	: inout std_logic;  
	pin43	: inout std_logic;  
	pin44	: inout std_logic;
	pin45	: inout std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin51	: inout std_logic;  
	pin52	: inout std_logic;  
	pin53	: inout std_logic;  
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: in std_logic;
	pin63	: inout std_logic;  
	pin64	: inout std_logic;
	pin65	: inout std_logic;
	pin66	: inout std_logic;
	pin67	: inout std_logic;
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin72	: inout std_logic;  
	pin73	: inout std_logic;  
	pin74	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: inout std_logic;
	pin79	: inout std_logic;
	pin82	: inout std_logic;  
	pin83	: inout std_logic;  
	pin84	: inout std_logic;
	pin85	: inout std_logic;
	pin86	: inout std_logic;
	pin87	: inout std_logic;
	pin88	: inout std_logic;
	pin89	: inout std_logic;
	pin91	: inout std_logic;  
	pin92	: inout std_logic;  
	pin93	: inout std_logic;  
	pin94	: inout std_logic;
	pin95	: inout std_logic;
	pin96	: inout std_logic;
	pin97	: inout std_logic;
	pin98	: inout std_logic;
	pin99	: in std_logic;
	pin102	: in std_logic;  
	pin103	: inout std_logic;  
	pin104	: inout std_logic;
	pin105	: inout std_logic;
	pin106	: inout std_logic;
	pin107	: inout std_logic;
	pin108	: inout std_logic;
	pin109	: inout std_logic;
	pin110	: inout std_logic;
	pin112	: inout std_logic;  
	pin113	: inout std_logic;  
	pin114	: inout std_logic;
	pin115	: inout std_logic;
	pin116	: inout std_logic;
	pin117	: inout std_logic;
	pin118	: inout std_logic;
	pin119	: inout std_logic;
	pin122	: inout std_logic;  
	pin123	: inout std_logic;  
	pin124	: inout std_logic;
	pin125	: inout std_logic;
	pin126	: inout std_logic;
	pin127	: inout std_logic;
	pin128	: inout std_logic;
	pin129	: inout std_logic;
	pin131	: inout std_logic;  
	pin132	: inout std_logic;  
	pin133	: inout std_logic;  
	pin134	: inout std_logic;
	pin135	: inout std_logic;
	pin136	: inout std_logic;
	pin137	: inout std_logic;
	pin138	: inout std_logic;
	pin139	: in std_logic;
	pin143	: inout std_logic;  
	pin144	: inout std_logic;
	pin145	: inout std_logic;
	pin146	: inout std_logic;
	pin147	: inout std_logic;
	pin148	: inout std_logic;
	pin149	: inout std_logic;
	pin150	: inout std_logic;
	pin152	: inout std_logic;  
	pin153	: inout std_logic;  
	pin154	: inout std_logic;
	pin155	: inout std_logic;
	pin156	: inout std_logic;
	pin157	: inout std_logic;
	pin158	: inout std_logic;
	pin159	: inout std_logic);
end C375;

library ieee;
use ieee.std_logic_1164.all;

entity c376 is
    port (
	pin2	: inout std_logic;  
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin11	: inout std_logic;  
	pin12	: inout std_logic;  
	pin13	: inout std_logic;  
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: in std_logic;
	pin22	: in std_logic;
	pin23	: inout std_logic;  
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin32	: inout std_logic;  
	pin33	: inout std_logic;  
	pin34	: inout std_logic;
	pin35	: inout std_logic;
	pin36	: inout std_logic;
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin42	: inout std_logic;  
	pin43	: inout std_logic;  
	pin44	: inout std_logic;
	pin45	: inout std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin51	: inout std_logic;  
	pin52	: inout std_logic;  
	pin53	: inout std_logic;  
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: in std_logic;
	pin63	: inout std_logic;  
	pin64	: inout std_logic;
	pin65	: inout std_logic;
	pin66	: inout std_logic;
	pin67	: inout std_logic;
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin72	: inout std_logic;  
	pin73	: inout std_logic;  
	pin74	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: inout std_logic;
	pin79	: inout std_logic;
	pin82	: inout std_logic;  
	pin83	: inout std_logic;  
	pin84	: inout std_logic;
	pin85	: inout std_logic;
	pin86	: inout std_logic;
	pin87	: inout std_logic;
	pin88	: inout std_logic;
	pin89	: inout std_logic;
	pin91	: inout std_logic;  
	pin92	: inout std_logic;  
	pin93	: inout std_logic;  
	pin94	: inout std_logic;
	pin95	: inout std_logic;
	pin96	: inout std_logic;
	pin97	: inout std_logic;
	pin98	: inout std_logic;
	pin99	: in std_logic;
	pin102	: in std_logic;  
	pin103	: inout std_logic;  
	pin104	: inout std_logic;
	pin105	: inout std_logic;
	pin106	: inout std_logic;
	pin107	: inout std_logic;
	pin108	: inout std_logic;
	pin109	: inout std_logic;
	pin110	: inout std_logic;
	pin112	: inout std_logic;  
	pin113	: inout std_logic;  
	pin114	: inout std_logic;
	pin115	: inout std_logic;
	pin116	: inout std_logic;
	pin117	: inout std_logic;
	pin118	: inout std_logic;
	pin119	: inout std_logic;
	pin122	: inout std_logic;  
	pin123	: inout std_logic;  
	pin124	: inout std_logic;
	pin125	: inout std_logic;
	pin126	: inout std_logic;
	pin127	: inout std_logic;
	pin128	: inout std_logic;
	pin129	: inout std_logic;
	pin131	: inout std_logic;  
	pin132	: inout std_logic;  
	pin133	: inout std_logic;  
	pin134	: inout std_logic;
	pin135	: inout std_logic;
	pin136	: inout std_logic;
	pin137	: inout std_logic;
	pin138	: inout std_logic;
	pin139	: in std_logic;
	pin143	: inout std_logic;  
	pin144	: inout std_logic;
	pin145	: inout std_logic;
	pin146	: inout std_logic;
	pin147	: inout std_logic;
	pin148	: inout std_logic;
	pin149	: inout std_logic;
	pin150	: inout std_logic;
	pin152	: inout std_logic;  
	pin153	: inout std_logic;  
	pin154	: inout std_logic;
	pin155	: inout std_logic;
	pin156	: inout std_logic;
	pin157	: inout std_logic;
	pin158	: inout std_logic;
	pin159	: inout std_logic);
end C376;
